import os
import time

from redis import StrictRedis


REDIS_DB = os.environ['REDIS_DB']
REDIS_SERVER = os.environ['REDIS_SERVER']
REDIS_PREFIX = os.environ.get('REDIS_PREFIX', '')


class RateLimit(StrictRedis):
    def __init__(self, expire=None):
        super().__init__({'host': REDIS_SERVER, 'db': REDIS_DB})

    def current_milli_time(self):
        return int(round(time.time() * 1000))

    def check_limits(self, identifier, limit_type=None):
        """
        Verifica um limite específico e abuso em uma única chamada ao Redis.
        Se limit_type for None, usa "api" como padrão.
        Retorna {"rate_limited": bool, "abuse": bool, "retry_after": int}
        """
        now = self.current_milli_time()
        type_to_check = limit_type if limit_type else "api"

        # Configurações fixas
        configs = {
            "api": {"interval": 1000, "limit": 5, "key": f"{REDIS_PREFIX}:limits:api:{identifier}"},
            "login": {"interval": 5000, "limit": 3, "key": f"{REDIS_PREFIX}:limits:login:{identifier}"},
            "abuse": {"interval": 5000, "limit": 30, "key": f"{REDIS_PREFIX}:limits:requests:{identifier}"}
        }

        # Configuração do tipo solicitado
        config = configs.get(type_to_check, configs["api"])
        config_abuse = configs["abuse"]  # Sempre verifica abuso

        # Pipeline para operações do tipo específico e abuso
        p = self.pipeline()

        # Operações para o tipo específico
        p.zremrangebyscore(config["key"], 0, now - config["interval"])
        p.zadd(config["key"], {now: now})
        p.zcard(config["key"])
        p.zrange(config["key"], 0, 0, withscores=True)  # Pega o timestamp mais antigo
        p.expire(config["key"], int(config["interval"] / 1000))

        # Operações para abuso (sempre verificado)
        p.zremrangebyscore(config_abuse["key"], 0, now - config_abuse["interval"])
        p.zadd(config_abuse["key"], {now: now})
        p.zcard(config_abuse["key"])
        p.zrange(config_abuse["key"], 0, 0, withscores=True)  # Pega o timestamp mais antigo
        p.expire(config_abuse["key"], int(config_abuse["interval"] / 1000))

        pipeline_results = p.execute()

        # Processa resultados para o tipo específico
        count_type = pipeline_results[2]  # ZCARD do tipo específico
        oldest_type = pipeline_results[3]  # Timestamp mais antigo do tipo
        is_rate_limited = count_type > config["limit"]

        # Processa resultados para abuso
        count_abuse = pipeline_results[7]  # ZCARD do abuso
        oldest_abuse = pipeline_results[8]  # Timestamp mais antigo do abuso
        is_abuse = count_abuse > config_abuse["limit"]

        # Calcula retry_after dinamicamente
        retry_after = 0
        if is_rate_limited and oldest_type:
            oldest_time = int(float(oldest_type[0][1]))  # Converte para inteiro
            retry_after = max(0, (oldest_time + config["interval"]) - now)

        if is_abuse and oldest_abuse:
            oldest_abuse_time = int(float(oldest_abuse[0][1]))  # Converte para inteiro
            abuse_retry = max(0, (oldest_abuse_time + config_abuse["interval"]) - now)
            retry_after = max(retry_after, abuse_retry)

        return {
            "rate_limited": is_rate_limited,
            "abuse": is_abuse,
            "retry_after": retry_after
        }

    def api_limited(self, identifier):
        return self.check_limits(identifier, "api")

    def login_limited(self, identifier):
        return self.check_limits(identifier, "login")


RateLimiter = RateLimit()
